''' Resources for User Accounts '''

from pyramid.security import Allow, Authenticated, Deny, DENY_ALL, Everyone

from ordr2.models import User

from .base import BaseResource


class PasswordResetAccount(BaseResource):
    ''' resource for passwort change representing a reset token '''

    def __acl__(self):
        ''' access controll list '''
        return [
            (Allow, Everyone, 'reset'),
            DENY_ALL
            ]


class PasswordReset(BaseResource):
    ''' resource for passwort reset link '''

    def __acl__(self):
        ''' access controll list '''
        return [
            (Allow, Everyone, 'reset'),
            DENY_ALL
            ]

    def __getitem__(self, key):
        ''' queries the database for a password reset token '''
        key = key.strip()
        if key:
            account = self.request.dbsession.\
                query(User).\
                filter_by(password_reset=key).\
                first()
            if account:
                return PasswordResetAccount(key, self, account)
        raise KeyError


class Account(BaseResource):
    ''' User Account and Settings '''

    nodes = {'reset': PasswordReset}

    def __init__(self, name, parent):
        super().__init__(name, parent)
        self.model = self.request.user


    def __acl__(self):
        ''' access controll list '''
        return [
            (Allow, Everyone, 'login'),
            (Allow, Everyone, 'logout'),
            (Deny, Authenticated, 'register'),
            (Allow, Everyone, 'register'),
            (Allow, Authenticated, 'settings'),
            (Allow, Everyone, 'reset'),
            DENY_ALL
            ]
